/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Leo Merholz
 *      Pascal Schärli
 *   Co-authors:
 *      ...
 *
 */

/**
    @file FlappyOrxShip.h
    @brief Declaration of the FlappyOrxShip class.
*/

#ifndef _FlappyOrxShip_H__
#define _FlappyOrxShip_H__

#include "flappyorx/FlappyOrxPrereqs.h"

#include "weapons/WeaponsPrereqs.h"
#include "worldentities/pawns/SpaceShip.h"
#include "graphics/ParticleSpawner.h"
#include "FlappyOrx.h"


namespace orxonox
{
    class _FlappyOrxExport FlappyOrxShip : public SpaceShip
    {
        public:
            FlappyOrxShip(Context* context);

            virtual void tick(float dt) override;

            // Starts or stops fireing
            virtual void boost(bool bBoost) override;
            
            inline void  setSpeedBase(float speedBase){ getGame()->setSpeedBase(speedBase);}
            inline float getSpeedBase(){ return getGame()->getSpeedBase();}
            inline void  setSpeedIncrease(float speedIncrease){ getGame()->setSpeedIncrease(speedIncrease);}
            inline float getSpeedIncrease(){ return getGame()->getSpeedIncrease();}

            inline void setTubeDistanceBase(float tubeDistanceBase){ getGame()->setTubeDistanceBase(tubeDistanceBase);}
            inline float  getTubeDistanceBase(){ return getGame()->getTubeDistanceBase();}
            inline void setTubeDistanceIncrease(float tubeDistanceIncrease){ getGame()->setTubeDistanceIncrease(tubeDistanceIncrease);}
            inline float  getTubeDistanceIncrease(){ return getGame()->getTubeDistanceIncrease();}

            inline void  setUpwardthrust( float upwardThrust ){ this->upwardThrust = upwardThrust; }
            inline float getUpwardthrust(){ return this->upwardThrust; }
            inline void  setGravity( float gravity ){ this->gravity = gravity; }
            inline float getGravity(){ return this->gravity; }

            inline void setSpeed( float speed ){ this->speed = speed; }
            inline float getSpeed( ){ return this->speed; }
            
   
            virtual time_t timeUntilRespawn();
            
            virtual void XMLPort(Element& xmlelement, XMLPort::Mode mode);


        protected:
            virtual void death() override;
        private:
            FlappyOrx* getGame();
            WeakPtr<FlappyOrx> game;
            ParticleSpawner* particlespawner_;
            bool isFlapping;
            bool isDead;
            float velocity;
            float speed, upwardThrust, gravity;
            float particleAge, particleLifespan;
            time_t deathTime;

    };
}

#endif /* _FlappyOrxShip_H__ */
